// Spieler.cc: Implementierung der Klasse CSpieler.
//
//////////////////////////////////////////////////////////////////////

#include "stdio.h"
#include "iostream.h"
#include "Spieler.h"

CSpieler::CSpieler (string strName, CMeister * pMeister)
{
    m_pstrName = new string;
    *m_pstrName = strName;
    
    m_praumAkt = NULL;
    
    m_plistGegenstand = new CList<CGegenstand *>;      

    m_pMeister = pMeister;
}
   
CSpieler::~CSpieler ()
{
    delete m_pstrName;
    delete m_plistGegenstand;
}

string CSpieler::GetName ()
{
    return (*m_pstrName);
}

int CSpieler::TakeGegenstand (int iPos)
{
	CGegenstand * pTemp;
	int iError;
	
	if (!m_praumAkt)
		// Aktueller Raum nicht definiert
		return 1;

    iError = m_praumAkt->RemoveGegenstand(iPos, pTemp);
	if (iError)
		// Gegenstandsindex ausserhalb des gueltigen Bereichs
		return 2;

	m_plistGegenstand->AddData(pTemp);

	return 0;
}

int CSpieler::DropGegenstand (int iPos)
{
	CGegenstand * pTemp;
	int iError;

	if (!m_praumAkt)
		// Aktueller Raum nicht definiert
		return 1;

	iError = m_plistGegenstand->RemoveData (iPos, pTemp);
	if (!iError)
		// Gegenstandsindex ausserhalb des gueltigen Bereichs
		return 2;

	m_praumAkt->AddGegenstand(pTemp);

	return 0;
}

int CSpieler::LeaveRaum (int iPos)
{
	CRaum * pTemp;
	int iError;

	if (!m_praumAkt)
		// Aktueller Raum nicht definiert
		return 2;
	
	iError = m_praumAkt->GetRaum (iPos, pTemp);
	if (iError)
		// Raumindex ausserhalb des gueltigen Bereichs
		return 3;

	m_praumAkt = pTemp;

	return (m_pMeister->CheckEnde(m_praumAkt) ? 1 : 0);
}

void CSpieler::Start (CRaum * pStart, CList<CGegenstand *> * plistInventar)
{
	m_praumAkt = pStart;
	CGegenstand * pGegenstand;
	char ch;
	int loop = 1;
	
	while (plistInventar->RemoveData(1, pGegenstand))
		m_plistGegenstand->AddData(pGegenstand);
	
	cout << "\n\n\n\n";
	cout << "==========\n";
	cout << "SPIELSTART\n";
	cout << "==========\n";
	cout << "\n\n\n\n";
	
	while(loop)
	{
		cout << "Aktueller Raum: " << m_praumAkt->GetName() << "\n";
		cout << "[B]ewegen, [W]ege, [G]egenstaende, [I]nventar, [U]msehen, [N]ehmen, [A]blegen, [E]nde\n";
		cin >> ch;
		switch(toupper(ch))
		{
		case 'B':
			msgMove();
			if(m_pMeister->CheckEnde(m_praumAkt))
				loop = 0;
			break;
		case 'W':
			msgShowWege();
			break;
		case 'G':
			msgShowGegenstaende();
			break;
		case 'I':
			msgShowInventar();
			break;
		case 'U':
			msgLook();
			break;
		case 'N':
			msgTakeGegenstand();
			break;
		case 'A':
			msgDropGegenstand();
			break;
		case 'E':
			loop = 0;
			break;
		}
	}
	
	while (m_plistGegenstand->RemoveData(1, pGegenstand))
		plistInventar->AddData(pGegenstand);
	
}

void CSpieler::msgShowWege()
{
	int i = 0;
	int c = 1;
	char ch;
	CRaum * pRaum;
	
	cout << "\n";
	cout << "Verbluefft schaust Du Dich um und siehst folgende Wege:\n";

	while (!m_praumAkt->GetRaum(++i, pRaum))
	{
		cout << "   " << i << ": " << pRaum->GetName() << "\n";
		c++;
		if (c > 22)
		{
			cout << "Press almost any key to continue...\n";
			cin >> ch;
		}
	}
	cout << "\n";
}

void CSpieler::msgShowGegenstaende()
{
	int i = 0;
	int c = 1;
	char ch;
	CGegenstand * pGegenstand;
	
	cout << "\n";
	cout << "Verbluefft schaust Du Dich um und siehst folgende Gegenstaende:\n";

	while (!m_praumAkt->GetGegenstand(++i, pGegenstand))
	{
		cout << "   " << i << ": " << pGegenstand->GetName() << "\n";
		c++;
		if (c > 22)
		{
			cout << "Press almost any key to continue...\n";
			cin >> ch;
		}
	}
	cout << "\n";
}

void CSpieler::msgShowInventar()
{
	int i = 0;
	int c = 1;
	char ch;
	CGegenstand * pGegenstand;
	
	cout << "\n";
	cout << "Du findest in Deinen Taschen folgende Gegenstaende:\n";

	while (m_plistGegenstand->GetData(++i, pGegenstand))
	{
		cout << "   " << i << ": " << pGegenstand->GetName() << "\n";
		c++;
		if (c > 22)
		{
			cout << "Press almost any key to continue...\n";
			cin >> ch;
		}
	}
	cout << "\n";
}


void CSpieler::msgMove()
{
	int i;
	
	msgShowWege();

	cout << "Taxi - Scharia.... Wo Du wollen ?";
	cin >> i;
	cout << "\n";

	switch(LeaveRaum (i))
	{
	case 3:
	case 2:
		cout << "Zentrale, hier Uetzwurscht. Wo sein dieses Raum.";
		break;
	case 1:
	  // Spiel gewonnen!
		cout << "Elvis leben! Wir fahre Memphis!";
		break;
	case 0:
		cout << "Nix Memphis!";
		break;
	}
}

void CSpieler::msgLook()
{
	char ch;
	msgShowWege();
	cout << "\nPress almost any key to continue...\n";
	cin >> ch;
	msgShowGegenstaende();
	cout << "\nPress almost any key to continue...\n";
	cin >> ch;
	msgShowInventar();
	cout << "\nPress almost any key to continue...\n";
	cin >> ch;
}

void CSpieler::msgTakeGegenstand()
{
	int i;

	msgShowGegenstaende();
	
	cout << "Welchen Gegenstand soll ich aufheben ?";
	cin >> i;
	cout << "\n";

	switch(TakeGegenstand (i))
	{
	case 2:
	case 1:
		cout << "Diesen Gegenstand gibt es nicht!";
		break;
	case 0:
		cout << "OKai";
		break;
	}
}

void CSpieler::msgDropGegenstand()
{
	int i;

	msgShowInventar();
	
	cout << "Welchen Gegenstand soll ich ablegen ?";
	cin >> i;
	cout << "\n";

	switch(DropGegenstand (i))
	{
	case 2:
	case 1:
		cout << "Diesen Gegenstand gibt es nicht!";
		break;
	case 0:
		cout << "OKai";
		break;
	}
}
